/**********************************************************************************************/
/* FBL_Array.h                                                     					  	 	  */
/*                                                                       					  */
/* Foundation Base Library ( FBL )		                                   					  */
/*                                                                       					  */
/* Copyright Paradigma, 1993-2002															  */
/* All Rights Reserved                                                   					  */
/**********************************************************************************************/

#ifndef __FBL_Array_h
	#define __FBL_Array_h

#if VSDK_FRAMEWORK_STYLE
#	include <VSDK/FBL.h>
#else
#	include <FBL.h>
#endif // VSDK_FRAMEWORK_STYLE

#include <string.h>

#if defined(__MWERKS__)
	#pragma options align=power
#endif


/**********************************************************************************************/
template<class T> class FBL_Array
{
	// NOTE: range of this class is 1-based.

	public://///////////////////////////////////////////////////////////////////////////////////
 
				FBL_Array( 
					bool	inOwnsItems = false,
					ulong	inMaxItems	= 10 ) 
				{ 
					mData 		= NULL;
					mItems		= 0;
					mMaxItems	= inMaxItems;
					mOwnsItems	= inOwnsItems; 		

					if( mMaxItems ){
						mData = new T[mMaxItems];
						memset( (char*)mData, 0, mMaxItems * sizeof(T*) );
					}
				}
										
				FBL_Array( const FBL_Array& inSrc );						
										
virtual 		~FBL_Array( void ) { 
					if( mOwnsItems )
						DisposeItems();

					if( mData )
						delete [] mData;
				}

		void	DisposeItems( void );
		
		FBL_Array*	Clone( void );
					// make copy of this array on the heap and clone its items. 

		/* Methodes for items manipulations: */

		void	AddItem( T inItem ) { 
					if( mItems == mMaxItems )
						Resize( mMaxItems ? mMaxItems * 2 : 10 );

					mData[mItems++] = inItem;
				}
		
		void	SetArrayItem( T inItem, ulong inPos ) { 
					CHECK_FBL( 1 <= inPos && inPos <= mItems );
					mData[inPos-1] = inItem; 		
				}
										
		void 	InsertAfter( T inNewItem, ulong inAfterIndex );

		void 	InsertAfter( T inNewItem, T inAfterItem )
				{
					// if inAfterItem is not found then we append a new item
					// to the end of the array.
					ulong afterIndex = inAfterItem ? FindIndex(inAfterItem) : mItems; 
					InsertAfter( inNewItem, afterIndex );
				}


		void	RemoveItem( ulong inPos, bool inDontDestroy = false );	
							
		void	RemoveItem( T inItem, bool inDontDestroy = false ){ 
					ulong index = FindIndex(inItem);
					if( index > 0 )
						RemoveItem( index, inDontDestroy );	
				}		

		T		GetItemAt( ulong inPos ) const { 
					CHECK_FBL( 1 <= inPos && inPos <= mItems ); 
					if( 1 <= inPos && inPos <= mItems )
							return mData[inPos - 1];	
					else 	return NULL;	
				}
				
		T 		LastItem( void ) const { 
					return mItems ? mData[mItems-1] : NULL; 	
				}
		
		ulong	FindIndex( T inItem ) const ; 
		bool	Include( T inItem ) const { return bool(FindIndex(inItem) > 0); }		
	
		/* Accessories: */

		ulong 	GetCount( void ) const			{ return mItems; 		}	
		void	SetCount( ulong inCount )		{ mItems = inCount; 	} 
		
		ulong 	GetMaxCount( void ) const 		{ return mMaxItems;		}
				
		void	Resize( ulong inNewSize );

		bool	GetOwnsItem( void ) const		{ return mOwnsItems;	}
		void	SetOwnsItem( bool inOwns )		{ mOwnsItems = inOwns;	}
	
	protected:////////////////////////////////////////////////////////////////////////////////// 

		T*		mData;

		ulong	mItems;
		ulong	mMaxItems;

		bool	mOwnsItems;						
}; 


/**********************************************************************************************/
template <class T>
FBL_Array<T>::FBL_Array( const FBL_Array<T>& inSrc )
{
	mOwnsItems 	= false;
	mItems		= inSrc.mItems;
	mMaxItems	= inSrc.mMaxItems;

	mData = new T [mMaxItems];

	for( ulong i = 1; i <= mItems; ++i )
		mData[i-1] = inSrc.GetItemAt(i);

}


/**********************************************************************************************/
template <class T>
void FBL_Array<T>::DisposeItems( void )	
{ 
	for( ushort i = 0; i < mItems ; ++i ){
		T item 	= mData[i];	
		mData[i] = NULL;	// set pointer to the object in the array to NULL.
		
		delete item; // call destructor for item;
	}
	
	mItems = 0;
}

/**********************************************************************************************/
template <class T>
void FBL_Array<T>::InsertAfter( T inNewItem, ulong inAfterIndex )
{
	if( !(0 <= inAfterIndex && inAfterIndex <= mItems) )
		inAfterIndex = mItems;

	if( mItems == mMaxItems )
		Resize( mMaxItems ? mMaxItems * 2 : 10  );

	FBL_memmove( mData + inAfterIndex + 1, 
			 mData + inAfterIndex, (mItems - inAfterIndex) * sizeof(T*));

	mData[inAfterIndex] = inNewItem;
	mItems++;
}



/**********************************************************************************************/
template <class T>
void FBL_Array<T>::RemoveItem( ulong pos, bool inDontDestroy )	
{ 
	CHECK_FBL( 1 <= pos && pos <= mItems  ); 

	if( pos == 0 || pos > mItems )
		return;	// nothing to remove

	if( mOwnsItems && inDontDestroy == false ){	
		T pItem = mData[pos-1];
		delete pItem;
	}

	FBL_memmove( mData + pos - 1, mData + pos, (mItems - pos) * sizeof(T*) );
	mItems--;

}				


/**********************************************************************************************/
template<class T>
ulong FBL_Array<T>::FindIndex( T inItem ) const
{
	T* pItem	= mData;
	T* pEndItem	= mData + mItems;

	while( pItem < pEndItem )
	{
		if( *pItem == inItem )
			return (ulong)(pItem - mData + 1);

		++pItem;
	}

	return 0L;	// bad index.
}


/**********************************************************************************************/
template<class T>
void FBL_Array<T>::Resize( ulong inNewSize )
{
	if( inNewSize == 0 && mData )
	{
		if( mOwnsItems )
			DisposeItems();
	
		delete [] mData;
		mData = NULL;
	
		mItems = 0;
		mMaxItems = 0;
	}
	else
	{
		T* pNewArray = new T [inNewSize];
		ulong Count = mItems < inNewSize ? mItems : inNewSize; 

		if( mData )
		{
			FBL_memcpy( (char*)pNewArray, (const char*)mData, Count * sizeof(T*) ); 
			delete [] mData;
			mData = NULL;
		}

		// fill by zero rest items if any.
		memset( (char*)(pNewArray + Count), 0, (inNewSize - Count) * sizeof(T*) ); 

		mData = pNewArray;
		mMaxItems = inNewSize;
	}
}



/**********************************************************************************************/
#if defined(__MWERKS__)
	#pragma options align=reset
#endif

#endif